/*
 * project.c: handle operations on projects
 *
 * This module should be the only one to use the functions from titraxDB. 
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 *
 * $Id: project.c,v 1.4 1999/07/29 19:44:59 ga58 Exp $
 *
 * $Log: project.c,v $
 * Revision 1.4  1999/07/29 19:44:59  ga58
 * v3.35 modified shift and change algorithm
 *
 * Revision 1.3  1999/06/03 12:04:42  ga58
 *  - Version 3.32
 *
 * Revision 1.2  1999/02/28 16:36:43  ga58
 *  - Version 3.31
 *
 * Revision 1.1  1999/02/21 13:41:57  ga58
 *  - Version 3.29
 *
 * Revision 1.5  1999/02/14 17:58:46  ga58
 *  - Version: 3.28
 *
 * Revision 1.4  1999/01/31 19:05:10  ga58
 *  - Version 3.25. New layout for the time boxes and version info for the
 *    databases.
 *
 * Revision 1.3  1999/01/30 19:08:17  ga58
 *  - Version 3.24. Dynamically resizable number of project slots.
 *
 * Revision 1.2  1999/01/24 20:22:59  ga58
 *  - Beta version 3.23
 *
 * Revision 1.1  1999/01/17 20:02:22  ga58
 *  - nearly complete rewrite of titrax and all its modules
 *
 */

#include "titraxAll.h"
#include "slotData.h"
#include "slotName.h"
#include "slotNote.h"
#include "titraxID.h"
#include "titraxConst.h"
#include "titraxPref.h"
#include "datedb.h"
#include "project.h"

/*
 * For the purpose of this module, each project consists of three fields from TitraxDB:
 *
 * 1. The project name or the slot-number
 * 2. The time, when the project has been started
 * 3. The total elapsed time, before the project has been started
 *
 * For non-active projects only name and elapsed time are relevant. When dealing with the
 * active project we have to consider elapsed and the time difference between StartedAt and 
 * the current time.
 */
 
ULong
LastRunElapsed(Int slot)
{
  ULong now = TimGetSeconds();

  if ((slot == noSlotActive) || (slot != ActiveSlot()))
	 return 0;

  if (SlotData(slot)->startedAt > now) {
	 (void)FrmAlert(titraxTimeChangedAlert);
	 SetSlotData(slot, StartedAt, &now, sizeof(now));
  }

  return now - SlotData(slot)->startedAt;
}

ULong 
TotalElapsed(Int slot)
{
  ErrFatalDisplayIf((slot < 0) || (slot >= Prefs()->maxSlot), "TotalElapsed");
  
  return SlotData(slot)->elapsed + LastRunElapsed(slot);
}

/*
 * This is the only point in titrax, where Ignored(slot) is used. Ignored projects are 
 * excluded from the sum.
 */
ULong
SumElapsed(void)
{
  Int slot;
  ULong sum = 0;
  
  for(slot = 0; slot < Prefs()->maxSlot; slot++)
	 if (!SlotData(slot)->ignored)
		sum += TotalElapsed(slot);
  
  return sum;
}

void
Stop(Int slot)
{
  Char str[16];
  ULong now;
  ULong thisRun;
  ULong totalTime;

  if ((slot == noSlotActive) || (slot != ActiveSlot()))
	 return;
  
  now = TimGetSeconds();
  thisRun = LastRunElapsed(slot);
  totalTime = TotalElapsed(slot);

  if (SlotData(slot)->logging != dontLog)
	 if (((thisRun > 180) && (thisRun < 3600L * 24L * 30L)) ||
		  ((thisRun > 3600L * 24L * 30L) && 
			(FrmCustomAlert(titraxLargeEntryAlert, StrIToA(str, thisRun / (3600L * 24L)), "", "") == 0)))
		AddAppointment(slot, SlotData(slot)->startedAt, now);
  
  SetSlotData(slot, Elapsed, &totalTime, sizeof(totalTime));
  SetActiveSlot(noSlotActive);
}

void
Start(Int slot)
{
  ULong now = TimGetSeconds();

  ErrFatalDisplayIf((slot < 0) || (slot >= Prefs()->maxSlot), "Start");

  if (slot == ActiveSlot())
	 return;

  if (ActiveSlot() != noSlotActive)
	 Stop(ActiveSlot());
  
  SetActiveSlot(slot);
  SetSlotData(slot, StartedAt, &now, sizeof(now));
}

Int
ActiveSlot(void)
{
  Int slot;

  for(slot = 0; slot < Prefs()->maxSlot; slot++)
	 if (SlotData(slot)->active)
		return slot;

  return noSlotActive;
}

void
SetActiveSlot(Int slot)
{
  Int i = ActiveSlot();
  Boolean val = false;

  if (i != noSlotActive)
	 SetSlotData(i, Active, &val, sizeof(val));

  val = true;

  if (slot == noSlotActive)
	 return;
  else
	 SetSlotData(slot, Active, &val, sizeof(val));
}

Boolean
SlotIsEmpty(Int slot)
{
  ErrFatalDisplayIf((slot < 0) || (slot >= Prefs()->maxSlot), "SlotIsEmpty");

  return SlotName(slot)[0] == 0;
}

Boolean
NoteIsEmpty(Int slot)
{
  ErrFatalDisplayIf((slot < 0) || (slot >= Prefs()->maxSlot), "NoteIsEmpty");

  return StrLen(SlotNote(slot)) == 0;
}

void
SwapSlots(Int slot1, Int slot2)
{
  ErrFatalDisplayIf((slot1 < 0) || (slot1 >= Prefs()->maxSlot), "SwapSlots");
  ErrFatalDisplayIf((slot2 < 0) || (slot2 >= Prefs()->maxSlot), "SwapSlots");

  SwapSlotData(slot1, slot2);
  SwapSlotName(slot1, slot2);
  SwapSlotNote(slot1, slot2);
}

/*
 * A positive amount means: increase the time of this project
 */
void 
ChangeTime(Int slot, Long amount)
{
  ErrFatalDisplayIf((slot < 0) || (slot >= Prefs()->maxSlot), "ChangeTime");

  if (amount == 0)
	 return;

  if (slot == ActiveSlot()) {
	 if (SlotData(slot)->logging != dontLog) {
		ULong val = SlotData(slot)->startedAt - amount;
		SetSlotData(slot, StartedAt, &val, sizeof(val));
	 }
	 else {
		ULong val = TotalElapsed(slot) + amount;
		ULong now = TimGetSeconds();
		SetSlotData(slot, Elapsed, &val, sizeof(val));
		SetSlotData(slot, StartedAt, &now, sizeof(now)); 
	 }
  }
  else {
	 ULong val = SlotData(slot)->elapsed + amount;
	 SetSlotData(slot, Elapsed, &val, sizeof(val));

	 if (SlotData(slot)->logging != dontLog) {
		ULong start = SlotData(ActiveSlot())->startedAt;
		if (ActiveSlot() != noSlotActive)
		  AddAppointment(slot, start, start + amount);
	 }
  }
}

/*
 * A positive amount means: increase the time of the slot to
 */
void
ShiftTime(Int from, Int to, Long amount)
{
  ErrFatalDisplayIf((from < 0) || (from >= Prefs()->maxSlot), "ShiftTime");
  ErrFatalDisplayIf((to < 0) || (to >= Prefs()->maxSlot), "ShiftTime");

  ChangeTime(from, -amount);
  ChangeTime(to, amount);
}

/*
 * The next two functions are the heart of the titrax shifting logic.
 */
void
CalculateMinMaxChangeAmount(Int slot, Long* minAmount, Long* maxAmount)
{
  Boolean DB;
  Long elapsed;

  ErrFatalDisplayIf((slot < 0) || (slot >= Prefs()->maxSlot), "CalculateMinMaxChangeAmount");

  elapsed = TotalElapsed(slot);
  DB = (SlotData(slot)->logging != dontLog);

  if (slot != ActiveSlot())
	 if (DB) {
		*minAmount = 0;
		*maxAmount = LastRunElapsed(ActiveSlot());
		return;
	 } else {
		*minAmount = -elapsed;
		*maxAmount = 0x7FFF0000 - elapsed;
		return;
	 }
  
  if (slot == ActiveSlot())
	 if (DB) {
		*minAmount = -LastRunElapsed(slot);
		*maxAmount = 0x7FFF0000 - elapsed;
		return;
	 } else {
		*minAmount = -elapsed;
		*maxAmount = 0x7FFF0000 - elapsed;
		return;
	 }
}

/*
 * Here we have to remember that amount is the amount of time, the project to
 * will get.
 */
void
CalculateMinMaxShiftAmount(Int from, Int to, Long* minAmount, Long* maxAmount)
{
  Long fromMin, fromMax, toMin, toMax;

  ErrFatalDisplayIf((from < 0) || (from >= Prefs()->maxSlot), "CalculateMinMaxShiftAmount");
  ErrFatalDisplayIf((to < 0) || (to >= Prefs()->maxSlot), "CalculateMinMaxShiftAmount");

  CalculateMinMaxChangeAmount(from, &fromMin, &fromMax);
  CalculateMinMaxChangeAmount(to, &toMin, &toMax);
  
  *minAmount = max(toMin, -fromMax);
  *maxAmount = min(toMax, -fromMin);
}
