/*
 * slotDBUtil: database utility routines used by all slot db modules.
 *
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 *
 * $Id: slotDBUtil.c,v 1.2 1999/02/28 16:36:46 ga58 Exp $
 *
 * $Log: slotDBUtil.c,v $
 * Revision 1.2  1999/02/28 16:36:46  ga58
 *  - Version 3.31
 *
 * Revision 1.1  1999/02/21 13:41:57  ga58
 *  - Version 3.29
 *
 * Revision 1.3  1999/01/31 19:05:14  ga58
 *  - Version 3.25. New layout for the time boxes and version info for the
 *    databases.
 *
 * Revision 1.2  1999/01/30 19:08:19  ga58
 *  - Version 3.24. Dynamically resizable number of project slots.
 *
 * Revision 1.1  1999/01/24 20:23:05  ga58
 *  - Beta version 3.23
 *
 */

#include "titraxAll.h"
#include "titraxConst.h"
#include "titraxPref.h"
#include "slotDBUtil.h"

/*
 * Internal function: Creates a new record. On return, the new records is unlocked
 * and released.
 */
void
CreateRecord(DmOpenRef db, UInt index, VoidHand* hand, VoidPtr* ptr, UInt recordLen)
{
  UInt idx = index;

  hand[index] = DmNewRecord(db, &idx, recordLen);
  if (hand[index] == NULL)
	 ErrDisplay("CreateRecord: DmNewRecord failed.");

  if (idx != index)
	 ErrDisplay("CreateRecord: index error.");

  ptr[index] = MemHandleLock(hand[index]);
  if (ptr[index] == NULL)
	 ErrDisplay("CreateRecord: MemHandleLock failed.");

  DmSet(ptr[index], 0, recordLen, 0);

  MemHandleUnlock(hand[index]);
  DmReleaseRecord(db, index, true);
}

/*
 * Internal function: Remove an existing database and create a new (empty) one.
 * On return, the database is open.
 */
void 
CreateDatabase(DmOpenRef* db, CharPtr dbName, ULong dbType, ULong appType, UInt version)
{
  UInt cardNo = 0;
  LocalID dbID;

  dbID = DmFindDatabase(cardNo, dbName);
  if (dbID != 0)
	 DmDeleteDatabase(cardNo, dbID);

  if (DmCreateDatabase(cardNo, dbName, appType, dbType, false) != 0)
	 ErrDisplay("CreateDatabase: DmCreateDatabase failed.");

  *db = DmOpenDatabaseByTypeCreator(dbType, appType, dmModeReadWrite);
  ErrFatalDisplayIf(*db == 0, "CreateDatabase: DmOpenDatabaseByTypeCreator failed.");

  /* now write the correct version info */
  if (DmOpenDatabaseInfo(*db, &dbID, NULL, NULL, &cardNo, NULL) != 0)
	 ErrDisplay("CreateDatabase: DmOpenDatabaseInfo failed.");

  if (DmSetDatabaseInfo(cardNo, dbID, NULL, NULL, &version, NULL, NULL, NULL, NULL, NULL,
								NULL, NULL, NULL) != 0)
	 ErrDisplay("CreateDatabase: DmSetDatabaseInfo failed.");
}

/*
 * open a database. If the db does not exists, create it. If a record does not exist,
 * create it. If there are too much records, delete them. So we can resize any of our
 * databases by simply closing it, changing the value of MaxSlot and reopen it.
 */
void
OpenDatabase(DmOpenRef* db, CharPtr dbName, ULong dbType, ULong appType, VoidHand* hand, VoidPtr* ptr, UInt recordLen, UInt version)
{
  Int i;
  UInt numRecs;
  UInt cardNo;
  LocalID dbID;
  UInt versionFound;

  *db = DmOpenDatabaseByTypeCreator(dbType, appType, dmModeReadWrite);
  if (*db == 0)
	 CreateDatabase(db, dbName, dbType, appType, version);
  else {
	 /* check the version number of the now open database */
	 if (DmOpenDatabaseInfo(*db, &dbID, NULL, NULL, &cardNo, NULL) != 0)
		ErrDisplay("OpenDatabase: DmOpenDatabase failed.");
	 if (DmDatabaseInfo(cardNo, dbID, NULL, NULL, &versionFound, NULL, NULL, NULL, NULL,
							  NULL, NULL, NULL, NULL) != 0)
		ErrDisplay("OpenDatabase: DmDatabaseInfo failed.");
	 /* 
	  * treat a given version number of zero specially: ignore version conflicts
	  * in this case.
	  */
	 if ((versionFound != version) && (version != 0)) {
		DmCloseDatabase(*db);
		CreateDatabase(db, dbName, dbType, appType, version);
	 }
  }
  
  numRecs = DmNumRecords(*db);
  
  /* if numRecs > MaxSlot, remove the records */
  if (numRecs > Prefs()->maxSlot)
	 for(i = numRecs - 1; i >= Prefs()->maxSlot; i--)
		if (DmRemoveRecord(*db, i) != 0)
		  ErrDisplay("OpenDatabase: DmRemoveRecord failed.");
  
  /* create missing records */
  if (numRecs < Prefs()->maxSlot)
	 for(i = numRecs; i < Prefs()->maxSlot; i++)
		CreateRecord(*db, i, hand, ptr, recordLen);
  
  /* get and lock all records */
  for(i = 0; i < Prefs()->maxSlot; i++) {
	 hand[i] = DmGetRecord(*db, i);
	 if (hand[i] == NULL)
		ErrDisplay("OpenDatabase: DmGetRecord failed.");
	 
	 ptr[i] = MemHandleLock(hand[i]);
	 if (ptr[i] == NULL)
		ErrDisplay("OpenDatabase: MemHandleLock failed.");
  }
}

void 
CloseDatabase(DmOpenRef db, VoidHand* hand)
{
  UInt i;

  for(i = 0; i < Prefs()->maxSlot; i++) {
	 MemHandleUnlock(hand[i]);
	 DmReleaseRecord(db, i, true);
  }

  DmCloseDatabase(db);

}
